# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_object import TrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_tracks import ManyTracks
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_saved_track_object import PagingSavedTrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.many_audio_features import ManyAudioFeatures
from spotifywebapiwithfixesandimprovementsfromsonallux.models.audio_features_object import AudioFeaturesObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.audio_analysis_object import AudioAnalysisObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.recommendations_object import RecommendationsObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class TracksController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(TracksController, self).__init__(config)

    def get_track(self,
                  id,
                  market=None):
        """Does a GET request to /tracks/{id}.

        Get Spotify catalog information for a single track identified by its
        unique Spotify ID.

        Args:
            id (str): The request template parameter.
            market (str, optional): The request query parameter.

        Returns:
            TrackObject: Response from the API. A track

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/tracks/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(TrackObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_several_tracks(self,
                           ids,
                           market=None):
        """Does a GET request to /tracks.

        Get Spotify catalog information for multiple tracks based on their
        Spotify IDs.

        Args:
            ids (str): The request query parameter.
            market (str, optional): The request query parameter.

        Returns:
            ManyTracks: Response from the API. A set of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/tracks')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyTracks.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_saved_tracks(self,
                               market=None,
                               limit=20,
                               offset=0):
        """Does a GET request to /me/tracks.

        Get a list of the songs saved in the current Spotify user's 'Your
        Music' library.

        Args:
            market (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingSavedTrackObject: Response from the API. Pages of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/tracks')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingSavedTrackObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def save_tracks_user(self,
                         ids,
                         body=None):
        """Does a PUT request to /me/tracks.

        Save one or more tracks to the current user's 'Your Music' library.

        Args:
            ids (str): The request query parameter.
            body (MeTracksRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Track saved

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/tracks')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def remove_tracks_user(self,
                           ids,
                           body=None):
        """Does a DELETE request to /me/tracks.

        Remove one or more tracks from the current user's 'Your Music' library.

        Args:
            ids (str): The request query parameter.
            body (MeTracksRequest1, optional): The request body parameter.

        Returns:
            void: Response from the API. Track removed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/tracks')
            .http_method(HttpMethodEnum.DELETE)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def check_users_saved_tracks(self,
                                 ids):
        """Does a GET request to /me/tracks/contains.

        Check if one or more tracks is already saved in the current Spotify
        user's 'Your Music' library.

        Args:
            ids (str): The request query parameter.

        Returns:
            List[bool]: Response from the API. Array of booleans

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/tracks/contains')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_several_audio_features(self,
                                   ids):
        """Does a GET request to /audio-features.

        Get audio features for multiple tracks based on their Spotify IDs.

        Args:
            ids (str): The request query parameter.

        Returns:
            ManyAudioFeatures: Response from the API. A set of audio features

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/audio-features')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(ManyAudioFeatures.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_audio_features(self,
                           id):
        """Does a GET request to /audio-features/{id}.

        Get audio feature information for a single track identified by its
        unique
        Spotify ID.

        Args:
            id (str): The request template parameter.

        Returns:
            AudioFeaturesObject: Response from the API. Audio features for one
                track

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/audio-features/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(AudioFeaturesObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_audio_analysis(self,
                           id):
        """Does a GET request to /audio-analysis/{id}.

        Get a low-level audio analysis for a track in the Spotify catalog. The
        audio analysis describes the track’s structure and musical content,
        including rhythm, pitch, and timbre.

        Args:
            id (str): The request template parameter.

        Returns:
            AudioAnalysisObject: Response from the API. Audio analysis for one
                track

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/audio-analysis/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(AudioAnalysisObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_recommendations(self,
                            limit=20,
                            market=None,
                            seed_artists=None,
                            seed_genres=None,
                            seed_tracks=None,
                            min_acousticness=None,
                            max_acousticness=None,
                            target_acousticness=None,
                            min_danceability=None,
                            max_danceability=None,
                            target_danceability=None,
                            min_duration_ms=None,
                            max_duration_ms=None,
                            target_duration_ms=None,
                            min_energy=None,
                            max_energy=None,
                            target_energy=None,
                            min_instrumentalness=None,
                            max_instrumentalness=None,
                            target_instrumentalness=None,
                            min_key=None,
                            max_key=None,
                            target_key=None,
                            min_liveness=None,
                            max_liveness=None,
                            target_liveness=None,
                            min_loudness=None,
                            max_loudness=None,
                            target_loudness=None,
                            min_mode=None,
                            max_mode=None,
                            target_mode=None,
                            min_popularity=None,
                            max_popularity=None,
                            target_popularity=None,
                            min_speechiness=None,
                            max_speechiness=None,
                            target_speechiness=None,
                            min_tempo=None,
                            max_tempo=None,
                            target_tempo=None,
                            min_time_signature=None,
                            max_time_signature=None,
                            target_time_signature=None,
                            min_valence=None,
                            max_valence=None,
                            target_valence=None):
        """Does a GET request to /recommendations.

        Recommendations are generated based on the available information for a
        given seed entity and matched against similar artists and tracks. If
        there is sufficient information about the provided seeds, a list of
        tracks will be returned together with pool size details.
        For artists and tracks that are very new or obscure there might not be
        enough data to generate a list of tracks.

        Args:
            limit (int, optional): The request query parameter. Example: 20
            market (str, optional): The request query parameter.
            seed_artists (str, optional): The request query parameter.
            seed_genres (str, optional): The request query parameter.
            seed_tracks (str, optional): The request query parameter.
            min_acousticness (float, optional): The request query parameter.
            max_acousticness (float, optional): The request query parameter.
            target_acousticness (float, optional): The request query parameter.
            min_danceability (float, optional): The request query parameter.
            max_danceability (float, optional): The request query parameter.
            target_danceability (float, optional): The request query parameter.
            min_duration_ms (int, optional): The request query parameter.
            max_duration_ms (int, optional): The request query parameter.
            target_duration_ms (int, optional): The request query parameter.
            min_energy (float, optional): The request query parameter.
            max_energy (float, optional): The request query parameter.
            target_energy (float, optional): The request query parameter.
            min_instrumentalness (float, optional): The request query
                parameter.
            max_instrumentalness (float, optional): The request query
                parameter.
            target_instrumentalness (float, optional): The request query
                parameter.
            min_key (int, optional): The request query parameter.
            max_key (int, optional): The request query parameter.
            target_key (int, optional): The request query parameter.
            min_liveness (float, optional): The request query parameter.
            max_liveness (float, optional): The request query parameter.
            target_liveness (float, optional): The request query parameter.
            min_loudness (float, optional): The request query parameter.
            max_loudness (float, optional): The request query parameter.
            target_loudness (float, optional): The request query parameter.
            min_mode (int, optional): The request query parameter.
            max_mode (int, optional): The request query parameter.
            target_mode (int, optional): The request query parameter.
            min_popularity (int, optional): The request query parameter.
            max_popularity (int, optional): The request query parameter.
            target_popularity (int, optional): The request query parameter.
            min_speechiness (float, optional): The request query parameter.
            max_speechiness (float, optional): The request query parameter.
            target_speechiness (float, optional): The request query parameter.
            min_tempo (float, optional): The request query parameter.
            max_tempo (float, optional): The request query parameter.
            target_tempo (float, optional): The request query parameter.
            min_time_signature (int, optional): The request query parameter.
            max_time_signature (int, optional): The request query parameter.
            target_time_signature (int, optional): The request query parameter.
            min_valence (float, optional): The request query parameter.
            max_valence (float, optional): The request query parameter.
            target_valence (float, optional): The request query parameter.

        Returns:
            RecommendationsObject: Response from the API. A set of
                recommendations

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/recommendations')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('market')
                         .value(market))
            .query_param(Parameter()
                         .key('seed_artists')
                         .value(seed_artists))
            .query_param(Parameter()
                         .key('seed_genres')
                         .value(seed_genres))
            .query_param(Parameter()
                         .key('seed_tracks')
                         .value(seed_tracks))
            .query_param(Parameter()
                         .key('min_acousticness')
                         .value(min_acousticness))
            .query_param(Parameter()
                         .key('max_acousticness')
                         .value(max_acousticness))
            .query_param(Parameter()
                         .key('target_acousticness')
                         .value(target_acousticness))
            .query_param(Parameter()
                         .key('min_danceability')
                         .value(min_danceability))
            .query_param(Parameter()
                         .key('max_danceability')
                         .value(max_danceability))
            .query_param(Parameter()
                         .key('target_danceability')
                         .value(target_danceability))
            .query_param(Parameter()
                         .key('min_duration_ms')
                         .value(min_duration_ms))
            .query_param(Parameter()
                         .key('max_duration_ms')
                         .value(max_duration_ms))
            .query_param(Parameter()
                         .key('target_duration_ms')
                         .value(target_duration_ms))
            .query_param(Parameter()
                         .key('min_energy')
                         .value(min_energy))
            .query_param(Parameter()
                         .key('max_energy')
                         .value(max_energy))
            .query_param(Parameter()
                         .key('target_energy')
                         .value(target_energy))
            .query_param(Parameter()
                         .key('min_instrumentalness')
                         .value(min_instrumentalness))
            .query_param(Parameter()
                         .key('max_instrumentalness')
                         .value(max_instrumentalness))
            .query_param(Parameter()
                         .key('target_instrumentalness')
                         .value(target_instrumentalness))
            .query_param(Parameter()
                         .key('min_key')
                         .value(min_key))
            .query_param(Parameter()
                         .key('max_key')
                         .value(max_key))
            .query_param(Parameter()
                         .key('target_key')
                         .value(target_key))
            .query_param(Parameter()
                         .key('min_liveness')
                         .value(min_liveness))
            .query_param(Parameter()
                         .key('max_liveness')
                         .value(max_liveness))
            .query_param(Parameter()
                         .key('target_liveness')
                         .value(target_liveness))
            .query_param(Parameter()
                         .key('min_loudness')
                         .value(min_loudness))
            .query_param(Parameter()
                         .key('max_loudness')
                         .value(max_loudness))
            .query_param(Parameter()
                         .key('target_loudness')
                         .value(target_loudness))
            .query_param(Parameter()
                         .key('min_mode')
                         .value(min_mode))
            .query_param(Parameter()
                         .key('max_mode')
                         .value(max_mode))
            .query_param(Parameter()
                         .key('target_mode')
                         .value(target_mode))
            .query_param(Parameter()
                         .key('min_popularity')
                         .value(min_popularity))
            .query_param(Parameter()
                         .key('max_popularity')
                         .value(max_popularity))
            .query_param(Parameter()
                         .key('target_popularity')
                         .value(target_popularity))
            .query_param(Parameter()
                         .key('min_speechiness')
                         .value(min_speechiness))
            .query_param(Parameter()
                         .key('max_speechiness')
                         .value(max_speechiness))
            .query_param(Parameter()
                         .key('target_speechiness')
                         .value(target_speechiness))
            .query_param(Parameter()
                         .key('min_tempo')
                         .value(min_tempo))
            .query_param(Parameter()
                         .key('max_tempo')
                         .value(max_tempo))
            .query_param(Parameter()
                         .key('target_tempo')
                         .value(target_tempo))
            .query_param(Parameter()
                         .key('min_time_signature')
                         .value(min_time_signature))
            .query_param(Parameter()
                         .key('max_time_signature')
                         .value(max_time_signature))
            .query_param(Parameter()
                         .key('target_time_signature')
                         .value(target_time_signature))
            .query_param(Parameter()
                         .key('min_valence')
                         .value(min_valence))
            .query_param(Parameter()
                         .key('max_valence')
                         .value(max_valence))
            .query_param(Parameter()
                         .key('target_valence')
                         .value(target_valence))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(RecommendationsObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ErrorObject(object):

    """Implementation of the 'ErrorObject' model.

    Attributes:
        status (int): The HTTP status code (also returned in the response
            header; see [Response Status
            Codes](/documentation/web-api/concepts/api-calls#response-status-co
            des) for more information).
        message (str): A short description of the cause of the error.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "status": 'status',
        "message": 'message'
    }

    def __init__(self,
                 status=None,
                 message=None):
        """Constructor for the ErrorObject class"""

        # Initialize members of the class
        self.status = status 
        self.message = message 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        status = dictionary.get("status") if dictionary.get("status") else None
        message = dictionary.get("message") if dictionary.get("message") else None
        # Return an object of this model
        return cls(status,
                   message)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'status={self.status!r}, '
                f'message={self.message!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'status={self.status!s}, '
                f'message={self.message!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from spotifywebapiwithfixesandimprovementsfromsonallux.models.private_user_object import PrivateUserObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.public_user_object import PublicUserObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.cursor_paged_artists import CursorPagedArtists
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_artist_object import PagingArtistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_track_object import PagingTrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.unauthorized_exception import UnauthorizedException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.forbidden_exception import ForbiddenException
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.too_many_requests_exception import TooManyRequestsException


class UsersController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(UsersController, self).__init__(config)

    def get_current_users_profile(self):
        """Does a GET request to /me.

        Get detailed profile information about the current user (including the
        current user's username).

        Returns:
            PrivateUserObject: Response from the API. A user

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me')
            .http_method(HttpMethodEnum.GET)
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PrivateUserObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_profile(self,
                          user_id):
        """Does a GET request to /users/{user_id}.

        Get public profile information about a Spotify user.

        Args:
            user_id (str): The request template parameter.

        Returns:
            PublicUserObject: Response from the API. A user

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/users/{user_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('user_id')
                            .value(user_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PublicUserObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def follow_playlist(self,
                        playlist_id,
                        body=None):
        """Does a PUT request to /playlists/{playlist_id}/followers.

        Add the current user as a follower of a playlist.

        Args:
            playlist_id (str): The request template parameter.
            body (PlaylistsFollowersRequest, optional): The request body
                parameter.

        Returns:
            void: Response from the API. Playlist followed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/followers')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def unfollow_playlist(self,
                          playlist_id):
        """Does a DELETE request to /playlists/{playlist_id}/followers.

        Remove the current user as a follower of a playlist.

        Args:
            playlist_id (str): The request template parameter.

        Returns:
            void: Response from the API. Playlist unfollowed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/followers')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .auth(Single('oauth_2_0'))
        ).execute()

    def get_followed(self,
                     mtype,
                     after=None,
                     limit=20):
        """Does a GET request to /me/following.

        Get the current user's followed artists.

        Args:
            mtype (ItemType1Enum): The request query parameter.
            after (str, optional): The request query parameter.
            limit (int, optional): The request query parameter. Example: 20

        Returns:
            CursorPagedArtists: Response from the API. A paged set of artists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/following')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('type')
                         .value(mtype))
            .query_param(Parameter()
                         .key('after')
                         .value(after))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CursorPagedArtists.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def follow_artists_users(self,
                             mtype,
                             ids,
                             body=None):
        """Does a PUT request to /me/following.

        Add the current user as a follower of one or more artists or other
        Spotify users.

        Args:
            mtype (ItemType2Enum): The request query parameter.
            ids (str): The request query parameter.
            body (MeFollowingRequest, optional): The request body parameter.

        Returns:
            void: Response from the API. Artist or user followed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/following')
            .http_method(HttpMethodEnum.PUT)
            .query_param(Parameter()
                         .key('type')
                         .value(mtype))
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def unfollow_artists_users(self,
                               mtype,
                               ids,
                               body=None):
        """Does a DELETE request to /me/following.

        Remove the current user as a follower of one or more artists or other
        Spotify users.

        Args:
            mtype (ItemType3Enum): The request query parameter.
            ids (str): The request query parameter.
            body (MeFollowingRequest1, optional): The request body parameter.

        Returns:
            void: Response from the API. Artist or user unfollowed

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/following')
            .http_method(HttpMethodEnum.DELETE)
            .query_param(Parameter()
                         .key('type')
                         .value(mtype))
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('oauth_2_0'))
        ).execute()

    def check_current_user_follows(self,
                                   mtype,
                                   ids):
        """Does a GET request to /me/following/contains.

        Check to see if the current user is following one or more artists or
        other Spotify users.

        Args:
            mtype (ItemType3Enum): The request query parameter.
            ids (str): The request query parameter.

        Returns:
            List[bool]: Response from the API. Array of booleans

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/following/contains')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('type')
                         .value(mtype))
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def check_if_user_follows_playlist(self,
                                       playlist_id,
                                       ids):
        """Does a GET request to /playlists/{playlist_id}/followers/contains.

        Check to see if one or more Spotify users are following a specified
        playlist.

        Args:
            playlist_id (str): The request template parameter.
            ids (str): The request query parameter.

        Returns:
            List[bool]: Response from the API. Array of booleans

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/playlists/{playlist_id}/followers/contains')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('playlist_id')
                            .value(playlist_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('ids')
                         .value(ids))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_top_artists(self,
                              time_range='medium_term',
                              limit=20,
                              offset=0):
        """Does a GET request to /me/top/artists.

        Get the current user's top artists based on calculated affinity.

        Args:
            time_range (str, optional): The request query parameter. Example:
                medium_term
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingArtistObject: Response from the API. Pages of artists

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/top/artists')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('time_range')
                         .value(time_range))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingArtistObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()

    def get_users_top_tracks(self,
                             time_range='medium_term',
                             limit=20,
                             offset=0):
        """Does a GET request to /me/top/tracks.

        Get the current user's top tracks based on calculated affinity.

        Args:
            time_range (str, optional): The request query parameter. Example:
                medium_term
            limit (int, optional): The request query parameter. Example: 20
            offset (int, optional): The request query parameter. Example: 0

        Returns:
            PagingTrackObject: Response from the API. Pages of tracks

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/me/top/tracks')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('time_range')
                         .value(time_range))
            .query_param(Parameter()
                         .key('limit')
                         .value(limit))
            .query_param(Parameter()
                         .key('offset')
                         .value(offset))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('oauth_2_0'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PagingTrackObject.from_dictionary)
            .local_error('401', 'Bad or expired token. This can happen if the user revoked a token or\nthe access token has expired. You should re-authenticate the user.\n', UnauthorizedException)
            .local_error('403', 'Bad OAuth request (wrong consumer key, bad nonce, expired\ntimestamp...). Unfortunately, re-authenticating the user won\'t help here.\n', ForbiddenException)
            .local_error('429', 'The app has exceeded its rate limits.\n', TooManyRequestsException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class ExplicitContentSettingsObject(object):

    """Implementation of the 'ExplicitContentSettingsObject' model.

    Attributes:
        filter_enabled (bool): When `true`, indicates that explicit content
            should not be played.
        filter_locked (bool): When `true`, indicates that the explicit content
            setting is locked and can't be changed by the user.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "filter_enabled": 'filter_enabled',
        "filter_locked": 'filter_locked'
    }

    _optionals = [
        'filter_enabled',
        'filter_locked',
    ]

    def __init__(self,
                 filter_enabled=APIHelper.SKIP,
                 filter_locked=APIHelper.SKIP):
        """Constructor for the ExplicitContentSettingsObject class"""

        # Initialize members of the class
        if filter_enabled is not APIHelper.SKIP:
            self.filter_enabled = filter_enabled 
        if filter_locked is not APIHelper.SKIP:
            self.filter_locked = filter_locked 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        filter_enabled = dictionary.get("filter_enabled") if "filter_enabled" in dictionary.keys() else APIHelper.SKIP
        filter_locked = dictionary.get("filter_locked") if "filter_locked" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(filter_enabled,
                   filter_locked)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'filter_enabled={(self.filter_enabled if hasattr(self, "filter_enabled") else None)!r}, '
                f'filter_locked={(self.filter_locked if hasattr(self, "filter_locked") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'filter_enabled={(self.filter_enabled if hasattr(self, "filter_enabled") else None)!s}, '
                f'filter_locked={(self.filter_locked if hasattr(self, "filter_locked") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Server
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from spotifywebapiwithfixesandimprovementsfromsonallux.http.http_method_enum import HttpMethodEnum
from spotifywebapiwithfixesandimprovementsfromsonallux.models.o_auth_token import OAuthToken
from spotifywebapiwithfixesandimprovementsfromsonallux.exceptions.o_auth_provider_exception import OAuthProviderException


class OAuthAuthorizationController(BaseController):

    """A Controller to access Endpoints in the spotifywebapiwithfixesandimprovementsfromsonallux API."""
    def __init__(self, config):
        super(OAuthAuthorizationController, self).__init__(config)

    def request_token(self,
                      authorization,
                      code,
                      redirect_uri,
                      _optional_form_parameters=None):
        """Does a POST request to /api/token.

        Create a new OAuth 2 token.

        Args:
            authorization (str): Authorization header in Basic auth format
            code (str): Authorization Code
            redirect_uri (str): Redirect Uri
            _optional_form_parameters (Array, optional): Additional optional
                form parameters are supported by this endpoint

        Returns:
            OAuthToken: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.AUTH_SERVER)
            .path('/api/token')
            .http_method(HttpMethodEnum.POST)
            .form_param(Parameter()
                        .key('grant_type')
                        .value('authorization_code'))
            .header_param(Parameter()
                          .key('Authorization')
                          .value(authorization))
            .form_param(Parameter()
                        .key('code')
                        .value(code))
            .form_param(Parameter()
                        .key('redirect_uri')
                        .value(redirect_uri))
            .header_param(Parameter()
                          .key('content-type')
                          .value('application/x-www-form-urlencoded'))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .additional_form_params(_optional_form_parameters)
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(OAuthToken.from_dictionary)
            .local_error('400', 'OAuth 2 provider returned an error.', OAuthProviderException)
            .local_error('401', 'OAuth 2 provider says client authentication failed.', OAuthProviderException)
        ).execute()

    def refresh_token(self,
                      authorization,
                      refresh_token,
                      scope=None,
                      _optional_form_parameters=None):
        """Does a POST request to /api/token.

        Obtain a new access token using a refresh token

        Args:
            authorization (str): Authorization header in Basic auth format
            refresh_token (str): Refresh token
            scope (str, optional): Requested scopes as a space-delimited list.
            _optional_form_parameters (Array, optional): Additional optional
                form parameters are supported by this endpoint

        Returns:
            OAuthToken: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.AUTH_SERVER)
            .path('/api/token')
            .http_method(HttpMethodEnum.POST)
            .form_param(Parameter()
                        .key('grant_type')
                        .value('refresh_token'))
            .header_param(Parameter()
                          .key('Authorization')
                          .value(authorization))
            .form_param(Parameter()
                        .key('refresh_token')
                        .value(refresh_token))
            .form_param(Parameter()
                        .key('scope')
                        .value(scope))
            .header_param(Parameter()
                          .key('content-type')
                          .value('application/x-www-form-urlencoded'))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .additional_form_params(_optional_form_parameters)
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(OAuthToken.from_dictionary)
            .local_error('400', 'OAuth 2 provider returned an error.', OAuthProviderException)
            .local_error('401', 'OAuth 2 provider says client authentication failed.', OAuthProviderException)
        ).execute()



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class PlaylistsFollowersRequest(object):

    """Implementation of the 'Playlists Followers Request' model.

    Attributes:
        public (bool): Defaults to `true`. If `true` the playlist will be
            included in user's public playlists, if `false` it will remain
            private.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "public": 'public'
    }

    _optionals = [
        'public',
    ]

    def __init__(self,
                 public=APIHelper.SKIP):
        """Constructor for the PlaylistsFollowersRequest class"""

        # Initialize members of the class
        if public is not APIHelper.SKIP:
            self.public = public 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        public = dictionary.get("public") if "public" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(public)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'public={(self.public if hasattr(self, "public") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'public={(self.public if hasattr(self, "public") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class PlaylistSnapshotId(object):

    """Implementation of the 'PlaylistSnapshotId' model.

    Attributes:
        snapshot_id (str): The model property of type str.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "snapshot_id": 'snapshot_id'
    }

    _optionals = [
        'snapshot_id',
    ]

    def __init__(self,
                 snapshot_id=APIHelper.SKIP):
        """Constructor for the PlaylistSnapshotId class"""

        # Initialize members of the class
        if snapshot_id is not APIHelper.SKIP:
            self.snapshot_id = snapshot_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        snapshot_id = dictionary.get("snapshot_id") if dictionary.get("snapshot_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(snapshot_id)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'snapshot_id={(self.snapshot_id if hasattr(self, "snapshot_id") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.models.image_object import ImageObject


class CategoryObject(object):

    """Implementation of the 'CategoryObject' model.

    Attributes:
        href (str): A link to the Web API endpoint returning full details of
            the category.
        icons (List[ImageObject]): The category icon, in various sizes.
        id (str): The [Spotify category
            ID](/documentation/web-api/concepts/spotify-uris-ids) of the
            category.
        name (str): The name of the category.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "href": 'href',
        "icons": 'icons',
        "id": 'id',
        "name": 'name'
    }

    def __init__(self,
                 href=None,
                 icons=None,
                 id=None,
                 name=None):
        """Constructor for the CategoryObject class"""

        # Initialize members of the class
        self.href = href 
        self.icons = icons 
        self.id = id 
        self.name = name 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        href = dictionary.get("href") if dictionary.get("href") else None
        icons = None
        if dictionary.get('icons') is not None:
            icons = [ImageObject.from_dictionary(x) for x in dictionary.get('icons')]
        id = dictionary.get("id") if dictionary.get("id") else None
        name = dictionary.get("name") if dictionary.get("name") else None
        # Return an object of this model
        return cls(href,
                   icons,
                   id,
                   name)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'href={self.href!r}, '
                f'icons={self.icons!r}, '
                f'id={self.id!r}, '
                f'name={self.name!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'href={self.href!s}, '
                f'icons={self.icons!s}, '
                f'id={self.id!s}, '
                f'name={self.name!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OAuthScopeEnum(object):

    """Implementation of the 'OAuth Scope' enum.

    OAuth 2 scopes supported by the API

    Attributes:
        APP-REMOTE-CONTROL: Communicate with the Spotify app on your device. 
        PLAYLIST-READ-PRIVATE: Access your private playlists. 
        PLAYLIST-READ-COLLABORATIVE: Access your collaborative playlists. 
        PLAYLIST-MODIFY-PUBLIC: Manage your public playlists. 
        PLAYLIST-MODIFY-PRIVATE: Manage your private playlists. 
        USER-LIBRARY-READ: Access your saved content. 
        USER-LIBRARY-MODIFY: Manage your saved content. 
        USER-READ-PRIVATE: Access your subscription details. 
        USER-READ-EMAIL: Get your real email address. 
        USER-FOLLOW-READ: Access your followers and who you are following. 
        USER-FOLLOW-MODIFY: Manage your saved content. 
        USER-TOP-READ: Read your top artists and content. 
        USER-READ-PLAYBACK-POSITION: Read your position in content you have
            played. 
        USER-READ-PLAYBACK-STATE: Read your currently playing content and
            Spotify Connect devices information. 
        USER-READ-RECENTLY-PLAYED: Access your recently played items. 
        USER-READ-CURRENTLY-PLAYING: Read your currently playing content. 
        USER-MODIFY-PLAYBACK-STATE: Control playback on your Spotify clients
            and Spotify Connect devices. 
        UGC-IMAGE-UPLOAD: Upload images to Spotify on your behalf. 
        STREAMING: Play content and control playback on your other devices. 

    """
    APP_REMOTE_CONTROL = 'app-remote-control'

    PLAYLIST_READ_PRIVATE = 'playlist-read-private'

    PLAYLIST_READ_COLLABORATIVE = 'playlist-read-collaborative'

    PLAYLIST_MODIFY_PUBLIC = 'playlist-modify-public'

    PLAYLIST_MODIFY_PRIVATE = 'playlist-modify-private'

    USER_LIBRARY_READ = 'user-library-read'

    USER_LIBRARY_MODIFY = 'user-library-modify'

    USER_READ_PRIVATE = 'user-read-private'

    USER_READ_EMAIL = 'user-read-email'

    USER_FOLLOW_READ = 'user-follow-read'

    USER_FOLLOW_MODIFY = 'user-follow-modify'

    USER_TOP_READ = 'user-top-read'

    USER_READ_PLAYBACK_POSITION = 'user-read-playback-position'

    USER_READ_PLAYBACK_STATE = 'user-read-playback-state'

    USER_READ_RECENTLY_PLAYED = 'user-read-recently-played'

    USER_READ_CURRENTLY_PLAYING = 'user-read-currently-playing'

    USER_MODIFY_PLAYBACK_STATE = 'user-modify-playback-state'

    UGC_IMAGE_UPLOAD = 'ugc-image-upload'

    STREAMING = 'streaming'




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ModeEnum(object):

    """Implementation of the 'Mode' enum.

    Indicates the modality (major or minor) of a section, the type of scale
    from which its melodic content is derived. This field will contain a 0 for
    "minor", a 1 for "major", or a -1 for no result. Note that the major key
    (e.g. C major) could more likely be confused with the minor key at 3
    semitones lower (e.g. A minor) as both keys carry the same pitches.

    Attributes:
        ENUM_MINUS1: The enum member of type int.
        ENUM_0: The enum member of type int.
        ENUM_1: The enum member of type int.

    """
    ENUM_MINUS1 = -1

    ENUM_0 = 0

    ENUM_1 = 1




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class SectionObject(object):

    """Implementation of the 'SectionObject' model.

    Attributes:
        start (float): The starting point (in seconds) of the section.
        duration (float): The duration (in seconds) of the section.
        confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the section's "designation".
        loudness (float): The overall loudness of the section in decibels
            (dB). Loudness values are useful for comparing relative loudness
            of sections within tracks.
        tempo (float): The overall estimated tempo of the section in beats per
            minute (BPM). In musical terminology, tempo is the speed or pace
            of a given piece and derives directly from the average beat
            duration.
        tempo_confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the tempo. Some tracks contain tempo changes or
            sounds which don't contain tempo (like pure speech) which would
            correspond to a low value in this field.
        key (int): The estimated overall key of the section. The values in
            this field ranging from 0 to 11 mapping to pitches using standard
            Pitch Class notation (E.g. 0 = C, 1 = C♯/D♭, 2 = D, and so on). If
            no key was detected, the value is -1.
        key_confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the key. Songs with many key changes may correspond
            to low values in this field.
        mode (ModeEnum): Indicates the modality (major or minor) of a section,
            the type of scale from which its melodic content is derived. This
            field will contain a 0 for "minor", a 1 for "major", or a -1 for
            no result. Note that the major key (e.g. C major) could more
            likely be confused with the minor key at 3 semitones lower (e.g. A
            minor) as both keys carry the same pitches.
        mode_confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the `mode`.
        time_signature (int): An estimated time signature. The time signature
            (meter) is a notational convention to specify how many beats are
            in each bar (or measure). The time signature ranges from 3 to 7
            indicating time signatures of "3/4", to "7/4".
        time_signature_confidence (float): The confidence, from 0.0 to 1.0, of
            the reliability of the `time_signature`. Sections with time
            signature changes may correspond to low values in this field.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "start": 'start',
        "duration": 'duration',
        "confidence": 'confidence',
        "loudness": 'loudness',
        "tempo": 'tempo',
        "tempo_confidence": 'tempo_confidence',
        "key": 'key',
        "key_confidence": 'key_confidence',
        "mode": 'mode',
        "mode_confidence": 'mode_confidence',
        "time_signature": 'time_signature',
        "time_signature_confidence": 'time_signature_confidence'
    }

    _optionals = [
        'start',
        'duration',
        'confidence',
        'loudness',
        'tempo',
        'tempo_confidence',
        'key',
        'key_confidence',
        'mode',
        'mode_confidence',
        'time_signature',
        'time_signature_confidence',
    ]

    def __init__(self,
                 start=APIHelper.SKIP,
                 duration=APIHelper.SKIP,
                 confidence=APIHelper.SKIP,
                 loudness=APIHelper.SKIP,
                 tempo=APIHelper.SKIP,
                 tempo_confidence=APIHelper.SKIP,
                 key=APIHelper.SKIP,
                 key_confidence=APIHelper.SKIP,
                 mode=APIHelper.SKIP,
                 mode_confidence=APIHelper.SKIP,
                 time_signature=APIHelper.SKIP,
                 time_signature_confidence=APIHelper.SKIP):
        """Constructor for the SectionObject class"""

        # Initialize members of the class
        if start is not APIHelper.SKIP:
            self.start = start 
        if duration is not APIHelper.SKIP:
            self.duration = duration 
        if confidence is not APIHelper.SKIP:
            self.confidence = confidence 
        if loudness is not APIHelper.SKIP:
            self.loudness = loudness 
        if tempo is not APIHelper.SKIP:
            self.tempo = tempo 
        if tempo_confidence is not APIHelper.SKIP:
            self.tempo_confidence = tempo_confidence 
        if key is not APIHelper.SKIP:
            self.key = key 
        if key_confidence is not APIHelper.SKIP:
            self.key_confidence = key_confidence 
        if mode is not APIHelper.SKIP:
            self.mode = mode 
        if mode_confidence is not APIHelper.SKIP:
            self.mode_confidence = mode_confidence 
        if time_signature is not APIHelper.SKIP:
            self.time_signature = time_signature 
        if time_signature_confidence is not APIHelper.SKIP:
            self.time_signature_confidence = time_signature_confidence 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        start = dictionary.get("start") if dictionary.get("start") else APIHelper.SKIP
        duration = dictionary.get("duration") if dictionary.get("duration") else APIHelper.SKIP
        confidence = dictionary.get("confidence") if dictionary.get("confidence") else APIHelper.SKIP
        loudness = dictionary.get("loudness") if dictionary.get("loudness") else APIHelper.SKIP
        tempo = dictionary.get("tempo") if dictionary.get("tempo") else APIHelper.SKIP
        tempo_confidence = dictionary.get("tempo_confidence") if dictionary.get("tempo_confidence") else APIHelper.SKIP
        key = dictionary.get("key") if dictionary.get("key") else APIHelper.SKIP
        key_confidence = dictionary.get("key_confidence") if dictionary.get("key_confidence") else APIHelper.SKIP
        mode = dictionary.get("mode") if dictionary.get("mode") else APIHelper.SKIP
        mode_confidence = dictionary.get("mode_confidence") if dictionary.get("mode_confidence") else APIHelper.SKIP
        time_signature = dictionary.get("time_signature") if dictionary.get("time_signature") else APIHelper.SKIP
        time_signature_confidence = dictionary.get("time_signature_confidence") if dictionary.get("time_signature_confidence") else APIHelper.SKIP
        # Return an object of this model
        return cls(start,
                   duration,
                   confidence,
                   loudness,
                   tempo,
                   tempo_confidence,
                   key,
                   key_confidence,
                   mode,
                   mode_confidence,
                   time_signature,
                   time_signature_confidence)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'start={(self.start if hasattr(self, "start") else None)!r}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!r}, '
                f'confidence={(self.confidence if hasattr(self, "confidence") else None)!r}, '
                f'loudness={(self.loudness if hasattr(self, "loudness") else None)!r}, '
                f'tempo={(self.tempo if hasattr(self, "tempo") else None)!r}, '
                f'tempo_confidence={(self.tempo_confidence if hasattr(self, "tempo_confidence") else None)!r}, '
                f'key={(self.key if hasattr(self, "key") else None)!r}, '
                f'key_confidence={(self.key_confidence if hasattr(self, "key_confidence") else None)!r}, '
                f'mode={(self.mode if hasattr(self, "mode") else None)!r}, '
                f'mode_confidence={(self.mode_confidence if hasattr(self, "mode_confidence") else None)!r}, '
                f'time_signature={(self.time_signature if hasattr(self, "time_signature") else None)!r}, '
                f'time_signature_confidence={(self.time_signature_confidence if hasattr(self, "time_signature_confidence") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'start={(self.start if hasattr(self, "start") else None)!s}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!s}, '
                f'confidence={(self.confidence if hasattr(self, "confidence") else None)!s}, '
                f'loudness={(self.loudness if hasattr(self, "loudness") else None)!s}, '
                f'tempo={(self.tempo if hasattr(self, "tempo") else None)!s}, '
                f'tempo_confidence={(self.tempo_confidence if hasattr(self, "tempo_confidence") else None)!s}, '
                f'key={(self.key if hasattr(self, "key") else None)!s}, '
                f'key_confidence={(self.key_confidence if hasattr(self, "key_confidence") else None)!s}, '
                f'mode={(self.mode if hasattr(self, "mode") else None)!s}, '
                f'mode_confidence={(self.mode_confidence if hasattr(self, "mode_confidence") else None)!s}, '
                f'time_signature={(self.time_signature if hasattr(self, "time_signature") else None)!s}, '
                f'time_signature_confidence={(self.time_signature_confidence if hasattr(self, "time_signature_confidence") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class AlbumGroupEnum(object):

    """Implementation of the 'AlbumGroup' enum.

    This field describes the relationship between the artist and the album.

    Attributes:
        ALBUM: The enum member of type str.
        SINGLE: The enum member of type str.
        COMPILATION: The enum member of type str.
        APPEARS_ON: The enum member of type str.

    """
    ALBUM = 'album'

    SINGLE = 'single'

    COMPILATION = 'compilation'

    APPEARS_ON = 'appears_on'




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.album_restriction_object import AlbumRestrictionObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.image_object import ImageObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.simplified_artist_object import SimplifiedArtistObject


class ArtistDiscographyAlbumObject(object):

    """Implementation of the 'ArtistDiscographyAlbumObject' model.

    Attributes:
        album_type (AlbumTypeEnum): The type of the album.
        total_tracks (int): The number of tracks in the album.
        available_markets (List[str]): The markets in which the album is
            available: [ISO 3166-1 alpha-2 country
            codes](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
            _**NOTE**: an album is considered available in a market when at
            least 1 of its tracks is available in that market._
        external_urls (ExternalUrlObject): Known external URLs for this album.
        href (str): A link to the Web API endpoint providing full details of
            the album.
        id (str): The [Spotify
            ID](/documentation/web-api/concepts/spotify-uris-ids) for the
            album.
        images (List[ImageObject]): The cover art for the album in various
            sizes, widest first.
        name (str): The name of the album. In case of an album takedown, the
            value may be an empty string.
        release_date (str): The date the album was first released.
        release_date_precision (ReleaseDatePrecisionEnum): The precision with
            which `release_date` value is known.
        restrictions (AlbumRestrictionObject): Included in the response when a
            content restriction is applied.
        mtype (str): The object type.
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for the
            album.
        artists (List[SimplifiedArtistObject]): The artists of the album. Each
            artist object includes a link in `href` to more detailed
            information about the artist.
        album_group (AlbumGroupEnum): This field describes the relationship
            between the artist and the album.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "album_type": 'album_type',
        "total_tracks": 'total_tracks',
        "available_markets": 'available_markets',
        "external_urls": 'external_urls',
        "href": 'href',
        "id": 'id',
        "images": 'images',
        "name": 'name',
        "release_date": 'release_date',
        "release_date_precision": 'release_date_precision',
        "mtype": 'type',
        "uri": 'uri',
        "artists": 'artists',
        "album_group": 'album_group',
        "restrictions": 'restrictions'
    }

    _optionals = [
        'restrictions',
    ]

    def __init__(self,
                 album_type=None,
                 total_tracks=None,
                 available_markets=None,
                 external_urls=None,
                 href=None,
                 id=None,
                 images=None,
                 name=None,
                 release_date=None,
                 release_date_precision=None,
                 uri=None,
                 artists=None,
                 album_group=None,
                 restrictions=APIHelper.SKIP):
        """Constructor for the ArtistDiscographyAlbumObject class"""

        # Initialize members of the class
        self.album_type = album_type 
        self.total_tracks = total_tracks 
        self.available_markets = available_markets 
        self.external_urls = external_urls 
        self.href = href 
        self.id = id 
        self.images = images 
        self.name = name 
        self.release_date = release_date 
        self.release_date_precision = release_date_precision 
        if restrictions is not APIHelper.SKIP:
            self.restrictions = restrictions 
        self.mtype = 'album' 
        self.uri = uri 
        self.artists = artists 
        self.album_group = album_group 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        album_type = dictionary.get("album_type") if dictionary.get("album_type") else None
        total_tracks = dictionary.get("total_tracks") if dictionary.get("total_tracks") else None
        available_markets = dictionary.get("available_markets") if dictionary.get("available_markets") else None
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if dictionary.get('external_urls') else None
        href = dictionary.get("href") if dictionary.get("href") else None
        id = dictionary.get("id") if dictionary.get("id") else None
        images = None
        if dictionary.get('images') is not None:
            images = [ImageObject.from_dictionary(x) for x in dictionary.get('images')]
        name = dictionary.get("name") if dictionary.get("name") else None
        release_date = dictionary.get("release_date") if dictionary.get("release_date") else None
        release_date_precision = dictionary.get("release_date_precision") if dictionary.get("release_date_precision") else None
        uri = dictionary.get("uri") if dictionary.get("uri") else None
        artists = None
        if dictionary.get('artists') is not None:
            artists = [SimplifiedArtistObject.from_dictionary(x) for x in dictionary.get('artists')]
        album_group = dictionary.get("album_group") if dictionary.get("album_group") else None
        restrictions = AlbumRestrictionObject.from_dictionary(dictionary.get('restrictions')) if 'restrictions' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(album_type,
                   total_tracks,
                   available_markets,
                   external_urls,
                   href,
                   id,
                   images,
                   name,
                   release_date,
                   release_date_precision,
                   uri,
                   artists,
                   album_group,
                   restrictions)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'album_type={self.album_type!r}, '
                f'total_tracks={self.total_tracks!r}, '
                f'available_markets={self.available_markets!r}, '
                f'external_urls={self.external_urls!r}, '
                f'href={self.href!r}, '
                f'id={self.id!r}, '
                f'images={self.images!r}, '
                f'name={self.name!r}, '
                f'release_date={self.release_date!r}, '
                f'release_date_precision={self.release_date_precision!r}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!r}, '
                f'mtype={self.mtype!r}, '
                f'uri={self.uri!r}, '
                f'artists={self.artists!r}, '
                f'album_group={self.album_group!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'album_type={self.album_type!s}, '
                f'total_tracks={self.total_tracks!s}, '
                f'available_markets={self.available_markets!s}, '
                f'external_urls={self.external_urls!s}, '
                f'href={self.href!s}, '
                f'id={self.id!s}, '
                f'images={self.images!s}, '
                f'name={self.name!s}, '
                f'release_date={self.release_date!s}, '
                f'release_date_precision={self.release_date_precision!s}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!s}, '
                f'mtype={self.mtype!s}, '
                f'uri={self.uri!s}, '
                f'artists={self.artists!s}, '
                f'album_group={self.album_group!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.types.union_types.leaf_type import LeafType
from apimatic_core.types.union_types.one_of import OneOf
from apimatic_core.types.union_types.union_type_context import UnionTypeContext as Context
from spotifywebapiwithfixesandimprovementsfromsonallux.models.episode_object import EpisodeObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_object import TrackObject


class UnionTypeLookUp:

    """The `UnionTypeLookUp` class serves as a utility class for 
    storing and managing type combinator templates.It acts as a container for the templates 
    used in handling various data types within the application.

    """
    _templates = {
        'PlaylistTrackObjectTrack': OneOf(
            [
                LeafType(TrackObject),
                LeafType(EpisodeObject)
            ],
            Context.create(
               is_optional=True
            )
        ),
        'QueueObjectCurrentlyPlaying': OneOf(
            [
                LeafType(TrackObject),
                LeafType(EpisodeObject)
            ],
            Context.create(
               is_optional=True
            )
        ),
        'QueueObjectQueue': OneOf(
            [
                LeafType(TrackObject),
                LeafType(EpisodeObject)
            ],
            Context.create(
               is_array=True,
               is_optional=True
            )
        ),
        'CurrentlyPlayingContextObjectItem': OneOf(
            [
                LeafType(TrackObject),
                LeafType(EpisodeObject)
            ],
            Context.create(
               is_optional=True
            )
        ),
        'CurrentlyPlayingObjectItem': OneOf(
            [
                LeafType(TrackObject),
                LeafType(EpisodeObject)
            ],
            Context.create(
               is_optional=True
            )
        )
    }

    @staticmethod
    def get(name):
        return UnionTypeLookUp._templates[name]




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.http.response.http_response import HttpResponse


class HttpResponse(HttpResponse):

    """Information about an HTTP Response including its status code, returned
        headers, and raw body

    Attributes:
        status_code (int): The status code response from the server that
            corresponds to this response.
        reason_phrase (string): The reason phrase returned by the server.
        headers (dict): A dictionary of headers (key : value) that were
            returned with the response
        text (string): The Raw body of the HTTP Response as a string
        request (HttpRequest): The request that resulted in this response.

    """

    def __init__(self,
                 status_code,
                 reason_phrase,
                 headers,
                 text,
                 request):
        """Constructor for the HttpResponse class

        Args:
            status_code (int): The response status code.
            reason_phrase (string): The response reason phrase.
            headers (dict): The response headers.
            text (string): The raw body from the server.
            request (HttpRequest): The request that resulted in this response.

        """
        super().__init__(status_code, reason_phrase, headers, text, request)



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class Track(object):

    """Implementation of the 'Track' model.

    Attributes:
        num_samples (int): The exact number of audio samples analyzed from
            this track. See also `analysis_sample_rate`.
        duration (float): Length of the track in seconds.
        sample_md_5 (str): This field will always contain the empty string.
        offset_seconds (int): An offset to the start of the region of the
            track that was analyzed. (As the entire track is analyzed, this
            should always be 0.)
        window_seconds (int): The length of the region of the track was
            analyzed, if a subset of the track was analyzed. (As the entire
            track is analyzed, this should always be 0.)
        analysis_sample_rate (int): The sample rate used to decode and analyze
            this track. May differ from the actual sample rate of this track
            available on Spotify.
        analysis_channels (int): The number of channels used for analysis. If
            1, all channels are summed together to mono before analysis.
        end_of_fade_in (float): The time, in seconds, at which the track's
            fade-in period ends. If the track has no fade-in, this will be 0.0.
        start_of_fade_out (float): The time, in seconds, at which the track's
            fade-out period starts. If the track has no fade-out, this should
            match the track's length.
        loudness (float): The overall loudness of a track in decibels (dB).
            Loudness values are averaged across the entire track and are
            useful for comparing relative loudness of tracks. Loudness is the
            quality of a sound that is the primary psychological correlate of
            physical strength (amplitude). Values typically range between -60
            and 0 db.
        tempo (float): The overall estimated tempo of a track in beats per
            minute (BPM). In musical terminology, tempo is the speed or pace
            of a given piece and derives directly from the average beat
            duration.
        tempo_confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the `tempo`.
        time_signature (int): An estimated time signature. The time signature
            (meter) is a notational convention to specify how many beats are
            in each bar (or measure). The time signature ranges from 3 to 7
            indicating time signatures of "3/4", to "7/4".
        time_signature_confidence (float): The confidence, from 0.0 to 1.0, of
            the reliability of the `time_signature`.
        key (int): The key the track is in. Integers map to pitches using
            standard [Pitch Class
            notation](https://en.wikipedia.org/wiki/Pitch_class). E.g. 0 = C,
            1 = C♯/D♭, 2 = D, and so on. If no key was detected, the value is
            -1.
        key_confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the `key`.
        mode (int): Mode indicates the modality (major or minor) of a track,
            the type of scale from which its melodic content is derived. Major
            is represented by 1 and minor is 0.
        mode_confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the `mode`.
        codestring (str): An [Echo Nest Musical Fingerprint
            (ENMFP)](https://academiccommons.columbia.edu/doi/10.7916/D8Q248M4)
            codestring for this track.
        code_version (float): A version number for the Echo Nest Musical
            Fingerprint format used in the codestring field.
        echoprintstring (str): An
            [EchoPrint](https://github.com/spotify/echoprint-codegen)
            codestring for this track.
        echoprint_version (float): A version number for the EchoPrint format
            used in the echoprintstring field.
        synchstring (str): A
            [Synchstring](https://github.com/echonest/synchdata) for this
            track.
        synch_version (float): A version number for the Synchstring used in
            the synchstring field.
        rhythmstring (str): A Rhythmstring for this track. The format of this
            string is similar to the Synchstring.
        rhythm_version (float): A version number for the Rhythmstring used in
            the rhythmstring field.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "num_samples": 'num_samples',
        "duration": 'duration',
        "sample_md_5": 'sample_md5',
        "offset_seconds": 'offset_seconds',
        "window_seconds": 'window_seconds',
        "analysis_sample_rate": 'analysis_sample_rate',
        "analysis_channels": 'analysis_channels',
        "end_of_fade_in": 'end_of_fade_in',
        "start_of_fade_out": 'start_of_fade_out',
        "loudness": 'loudness',
        "tempo": 'tempo',
        "tempo_confidence": 'tempo_confidence',
        "time_signature": 'time_signature',
        "time_signature_confidence": 'time_signature_confidence',
        "key": 'key',
        "key_confidence": 'key_confidence',
        "mode": 'mode',
        "mode_confidence": 'mode_confidence',
        "codestring": 'codestring',
        "code_version": 'code_version',
        "echoprintstring": 'echoprintstring',
        "echoprint_version": 'echoprint_version',
        "synchstring": 'synchstring',
        "synch_version": 'synch_version',
        "rhythmstring": 'rhythmstring',
        "rhythm_version": 'rhythm_version'
    }

    _optionals = [
        'num_samples',
        'duration',
        'sample_md_5',
        'offset_seconds',
        'window_seconds',
        'analysis_sample_rate',
        'analysis_channels',
        'end_of_fade_in',
        'start_of_fade_out',
        'loudness',
        'tempo',
        'tempo_confidence',
        'time_signature',
        'time_signature_confidence',
        'key',
        'key_confidence',
        'mode',
        'mode_confidence',
        'codestring',
        'code_version',
        'echoprintstring',
        'echoprint_version',
        'synchstring',
        'synch_version',
        'rhythmstring',
        'rhythm_version',
    ]

    def __init__(self,
                 num_samples=APIHelper.SKIP,
                 duration=APIHelper.SKIP,
                 sample_md_5=APIHelper.SKIP,
                 offset_seconds=APIHelper.SKIP,
                 window_seconds=APIHelper.SKIP,
                 analysis_sample_rate=APIHelper.SKIP,
                 analysis_channels=APIHelper.SKIP,
                 end_of_fade_in=APIHelper.SKIP,
                 start_of_fade_out=APIHelper.SKIP,
                 loudness=APIHelper.SKIP,
                 tempo=APIHelper.SKIP,
                 tempo_confidence=APIHelper.SKIP,
                 time_signature=APIHelper.SKIP,
                 time_signature_confidence=APIHelper.SKIP,
                 key=APIHelper.SKIP,
                 key_confidence=APIHelper.SKIP,
                 mode=APIHelper.SKIP,
                 mode_confidence=APIHelper.SKIP,
                 codestring=APIHelper.SKIP,
                 code_version=APIHelper.SKIP,
                 echoprintstring=APIHelper.SKIP,
                 echoprint_version=APIHelper.SKIP,
                 synchstring=APIHelper.SKIP,
                 synch_version=APIHelper.SKIP,
                 rhythmstring=APIHelper.SKIP,
                 rhythm_version=APIHelper.SKIP):
        """Constructor for the Track class"""

        # Initialize members of the class
        if num_samples is not APIHelper.SKIP:
            self.num_samples = num_samples 
        if duration is not APIHelper.SKIP:
            self.duration = duration 
        if sample_md_5 is not APIHelper.SKIP:
            self.sample_md_5 = sample_md_5 
        if offset_seconds is not APIHelper.SKIP:
            self.offset_seconds = offset_seconds 
        if window_seconds is not APIHelper.SKIP:
            self.window_seconds = window_seconds 
        if analysis_sample_rate is not APIHelper.SKIP:
            self.analysis_sample_rate = analysis_sample_rate 
        if analysis_channels is not APIHelper.SKIP:
            self.analysis_channels = analysis_channels 
        if end_of_fade_in is not APIHelper.SKIP:
            self.end_of_fade_in = end_of_fade_in 
        if start_of_fade_out is not APIHelper.SKIP:
            self.start_of_fade_out = start_of_fade_out 
        if loudness is not APIHelper.SKIP:
            self.loudness = loudness 
        if tempo is not APIHelper.SKIP:
            self.tempo = tempo 
        if tempo_confidence is not APIHelper.SKIP:
            self.tempo_confidence = tempo_confidence 
        if time_signature is not APIHelper.SKIP:
            self.time_signature = time_signature 
        if time_signature_confidence is not APIHelper.SKIP:
            self.time_signature_confidence = time_signature_confidence 
        if key is not APIHelper.SKIP:
            self.key = key 
        if key_confidence is not APIHelper.SKIP:
            self.key_confidence = key_confidence 
        if mode is not APIHelper.SKIP:
            self.mode = mode 
        if mode_confidence is not APIHelper.SKIP:
            self.mode_confidence = mode_confidence 
        if codestring is not APIHelper.SKIP:
            self.codestring = codestring 
        if code_version is not APIHelper.SKIP:
            self.code_version = code_version 
        if echoprintstring is not APIHelper.SKIP:
            self.echoprintstring = echoprintstring 
        if echoprint_version is not APIHelper.SKIP:
            self.echoprint_version = echoprint_version 
        if synchstring is not APIHelper.SKIP:
            self.synchstring = synchstring 
        if synch_version is not APIHelper.SKIP:
            self.synch_version = synch_version 
        if rhythmstring is not APIHelper.SKIP:
            self.rhythmstring = rhythmstring 
        if rhythm_version is not APIHelper.SKIP:
            self.rhythm_version = rhythm_version 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        num_samples = dictionary.get("num_samples") if dictionary.get("num_samples") else APIHelper.SKIP
        duration = dictionary.get("duration") if dictionary.get("duration") else APIHelper.SKIP
        sample_md_5 = dictionary.get("sample_md5") if dictionary.get("sample_md5") else APIHelper.SKIP
        offset_seconds = dictionary.get("offset_seconds") if dictionary.get("offset_seconds") else APIHelper.SKIP
        window_seconds = dictionary.get("window_seconds") if dictionary.get("window_seconds") else APIHelper.SKIP
        analysis_sample_rate = dictionary.get("analysis_sample_rate") if dictionary.get("analysis_sample_rate") else APIHelper.SKIP
        analysis_channels = dictionary.get("analysis_channels") if dictionary.get("analysis_channels") else APIHelper.SKIP
        end_of_fade_in = dictionary.get("end_of_fade_in") if dictionary.get("end_of_fade_in") else APIHelper.SKIP
        start_of_fade_out = dictionary.get("start_of_fade_out") if dictionary.get("start_of_fade_out") else APIHelper.SKIP
        loudness = dictionary.get("loudness") if dictionary.get("loudness") else APIHelper.SKIP
        tempo = dictionary.get("tempo") if dictionary.get("tempo") else APIHelper.SKIP
        tempo_confidence = dictionary.get("tempo_confidence") if dictionary.get("tempo_confidence") else APIHelper.SKIP
        time_signature = dictionary.get("time_signature") if dictionary.get("time_signature") else APIHelper.SKIP
        time_signature_confidence = dictionary.get("time_signature_confidence") if dictionary.get("time_signature_confidence") else APIHelper.SKIP
        key = dictionary.get("key") if dictionary.get("key") else APIHelper.SKIP
        key_confidence = dictionary.get("key_confidence") if dictionary.get("key_confidence") else APIHelper.SKIP
        mode = dictionary.get("mode") if dictionary.get("mode") else APIHelper.SKIP
        mode_confidence = dictionary.get("mode_confidence") if dictionary.get("mode_confidence") else APIHelper.SKIP
        codestring = dictionary.get("codestring") if dictionary.get("codestring") else APIHelper.SKIP
        code_version = dictionary.get("code_version") if dictionary.get("code_version") else APIHelper.SKIP
        echoprintstring = dictionary.get("echoprintstring") if dictionary.get("echoprintstring") else APIHelper.SKIP
        echoprint_version = dictionary.get("echoprint_version") if dictionary.get("echoprint_version") else APIHelper.SKIP
        synchstring = dictionary.get("synchstring") if dictionary.get("synchstring") else APIHelper.SKIP
        synch_version = dictionary.get("synch_version") if dictionary.get("synch_version") else APIHelper.SKIP
        rhythmstring = dictionary.get("rhythmstring") if dictionary.get("rhythmstring") else APIHelper.SKIP
        rhythm_version = dictionary.get("rhythm_version") if dictionary.get("rhythm_version") else APIHelper.SKIP
        # Return an object of this model
        return cls(num_samples,
                   duration,
                   sample_md_5,
                   offset_seconds,
                   window_seconds,
                   analysis_sample_rate,
                   analysis_channels,
                   end_of_fade_in,
                   start_of_fade_out,
                   loudness,
                   tempo,
                   tempo_confidence,
                   time_signature,
                   time_signature_confidence,
                   key,
                   key_confidence,
                   mode,
                   mode_confidence,
                   codestring,
                   code_version,
                   echoprintstring,
                   echoprint_version,
                   synchstring,
                   synch_version,
                   rhythmstring,
                   rhythm_version)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'num_samples={(self.num_samples if hasattr(self, "num_samples") else None)!r}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!r}, '
                f'sample_md_5={(self.sample_md_5 if hasattr(self, "sample_md_5") else None)!r}, '
                f'offset_seconds={(self.offset_seconds if hasattr(self, "offset_seconds") else None)!r}, '
                f'window_seconds={(self.window_seconds if hasattr(self, "window_seconds") else None)!r}, '
                f'analysis_sample_rate={(self.analysis_sample_rate if hasattr(self, "analysis_sample_rate") else None)!r}, '
                f'analysis_channels={(self.analysis_channels if hasattr(self, "analysis_channels") else None)!r}, '
                f'end_of_fade_in={(self.end_of_fade_in if hasattr(self, "end_of_fade_in") else None)!r}, '
                f'start_of_fade_out={(self.start_of_fade_out if hasattr(self, "start_of_fade_out") else None)!r}, '
                f'loudness={(self.loudness if hasattr(self, "loudness") else None)!r}, '
                f'tempo={(self.tempo if hasattr(self, "tempo") else None)!r}, '
                f'tempo_confidence={(self.tempo_confidence if hasattr(self, "tempo_confidence") else None)!r}, '
                f'time_signature={(self.time_signature if hasattr(self, "time_signature") else None)!r}, '
                f'time_signature_confidence={(self.time_signature_confidence if hasattr(self, "time_signature_confidence") else None)!r}, '
                f'key={(self.key if hasattr(self, "key") else None)!r}, '
                f'key_confidence={(self.key_confidence if hasattr(self, "key_confidence") else None)!r}, '
                f'mode={(self.mode if hasattr(self, "mode") else None)!r}, '
                f'mode_confidence={(self.mode_confidence if hasattr(self, "mode_confidence") else None)!r}, '
                f'codestring={(self.codestring if hasattr(self, "codestring") else None)!r}, '
                f'code_version={(self.code_version if hasattr(self, "code_version") else None)!r}, '
                f'echoprintstring={(self.echoprintstring if hasattr(self, "echoprintstring") else None)!r}, '
                f'echoprint_version={(self.echoprint_version if hasattr(self, "echoprint_version") else None)!r}, '
                f'synchstring={(self.synchstring if hasattr(self, "synchstring") else None)!r}, '
                f'synch_version={(self.synch_version if hasattr(self, "synch_version") else None)!r}, '
                f'rhythmstring={(self.rhythmstring if hasattr(self, "rhythmstring") else None)!r}, '
                f'rhythm_version={(self.rhythm_version if hasattr(self, "rhythm_version") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'num_samples={(self.num_samples if hasattr(self, "num_samples") else None)!s}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!s}, '
                f'sample_md_5={(self.sample_md_5 if hasattr(self, "sample_md_5") else None)!s}, '
                f'offset_seconds={(self.offset_seconds if hasattr(self, "offset_seconds") else None)!s}, '
                f'window_seconds={(self.window_seconds if hasattr(self, "window_seconds") else None)!s}, '
                f'analysis_sample_rate={(self.analysis_sample_rate if hasattr(self, "analysis_sample_rate") else None)!s}, '
                f'analysis_channels={(self.analysis_channels if hasattr(self, "analysis_channels") else None)!s}, '
                f'end_of_fade_in={(self.end_of_fade_in if hasattr(self, "end_of_fade_in") else None)!s}, '
                f'start_of_fade_out={(self.start_of_fade_out if hasattr(self, "start_of_fade_out") else None)!s}, '
                f'loudness={(self.loudness if hasattr(self, "loudness") else None)!s}, '
                f'tempo={(self.tempo if hasattr(self, "tempo") else None)!s}, '
                f'tempo_confidence={(self.tempo_confidence if hasattr(self, "tempo_confidence") else None)!s}, '
                f'time_signature={(self.time_signature if hasattr(self, "time_signature") else None)!s}, '
                f'time_signature_confidence={(self.time_signature_confidence if hasattr(self, "time_signature_confidence") else None)!s}, '
                f'key={(self.key if hasattr(self, "key") else None)!s}, '
                f'key_confidence={(self.key_confidence if hasattr(self, "key_confidence") else None)!s}, '
                f'mode={(self.mode if hasattr(self, "mode") else None)!s}, '
                f'mode_confidence={(self.mode_confidence if hasattr(self, "mode_confidence") else None)!s}, '
                f'codestring={(self.codestring if hasattr(self, "codestring") else None)!s}, '
                f'code_version={(self.code_version if hasattr(self, "code_version") else None)!s}, '
                f'echoprintstring={(self.echoprintstring if hasattr(self, "echoprintstring") else None)!s}, '
                f'echoprint_version={(self.echoprint_version if hasattr(self, "echoprint_version") else None)!s}, '
                f'synchstring={(self.synchstring if hasattr(self, "synchstring") else None)!s}, '
                f'synch_version={(self.synch_version if hasattr(self, "synch_version") else None)!s}, '
                f'rhythmstring={(self.rhythmstring if hasattr(self, "rhythmstring") else None)!s}, '
                f'rhythm_version={(self.rhythm_version if hasattr(self, "rhythm_version") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.linked_track_object import LinkedTrackObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.simplified_artist_object import SimplifiedArtistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.track_restriction_object import TrackRestrictionObject


class SimplifiedTrackObject(object):

    """Implementation of the 'SimplifiedTrackObject' model.

    Attributes:
        artists (List[SimplifiedArtistObject]): The artists who performed the
            track. Each artist object includes a link in `href` to more
            detailed information about the artist.
        available_markets (List[str]): A list of the countries in which the
            track can be played, identified by their [ISO 3166-1
            alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
        disc_number (int): The disc number (usually `1` unless the album
            consists of more than one disc).
        duration_ms (int): The track length in milliseconds.
        explicit (bool): Whether or not the track has explicit lyrics ( `true`
            = yes it does; `false` = no it does not OR unknown).
        external_urls (ExternalUrlObject): External URLs for this track.
        href (str): A link to the Web API endpoint providing full details of
            the track.
        id (str): The [Spotify
            ID](/documentation/web-api/concepts/spotify-uris-ids) for the
            track.
        is_playable (bool): Part of the response when [Track
            Relinking](/documentation/web-api/concepts/track-relinking/) is
            applied. If `true`, the track is playable in the given market.
            Otherwise `false`.
        linked_from (LinkedTrackObject): Part of the response when [Track
            Relinking](/documentation/web-api/concepts/track-relinking/) is
            applied and is only part of the response if the track linking, in
            fact, exists. The requested track has been replaced with a
            different track. The track in the `linked_from` object contains
            information about the originally requested track.
        restrictions (TrackRestrictionObject): Included in the response when a
            content restriction is applied.
        name (str): The name of the track.
        preview_url (str): A URL to a 30 second preview (MP3 format) of the
            track.
        track_number (int): The number of the track. If an album has several
            discs, the track number is the number on the specified disc.
        mtype (str): The object type: "track".
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for the
            track.
        is_local (bool): Whether or not the track is from a local file.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "artists": 'artists',
        "available_markets": 'available_markets',
        "disc_number": 'disc_number',
        "duration_ms": 'duration_ms',
        "explicit": 'explicit',
        "external_urls": 'external_urls',
        "href": 'href',
        "id": 'id',
        "is_playable": 'is_playable',
        "linked_from": 'linked_from',
        "restrictions": 'restrictions',
        "name": 'name',
        "preview_url": 'preview_url',
        "track_number": 'track_number',
        "mtype": 'type',
        "uri": 'uri',
        "is_local": 'is_local'
    }

    _optionals = [
        'artists',
        'available_markets',
        'disc_number',
        'duration_ms',
        'explicit',
        'external_urls',
        'href',
        'id',
        'is_playable',
        'linked_from',
        'restrictions',
        'name',
        'preview_url',
        'track_number',
        'mtype',
        'uri',
        'is_local',
    ]

    _nullables = [
        'preview_url',
    ]

    def __init__(self,
                 artists=APIHelper.SKIP,
                 available_markets=APIHelper.SKIP,
                 disc_number=APIHelper.SKIP,
                 duration_ms=APIHelper.SKIP,
                 explicit=APIHelper.SKIP,
                 external_urls=APIHelper.SKIP,
                 href=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 is_playable=APIHelper.SKIP,
                 linked_from=APIHelper.SKIP,
                 restrictions=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 preview_url=APIHelper.SKIP,
                 track_number=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 uri=APIHelper.SKIP,
                 is_local=APIHelper.SKIP):
        """Constructor for the SimplifiedTrackObject class"""

        # Initialize members of the class
        if artists is not APIHelper.SKIP:
            self.artists = artists 
        if available_markets is not APIHelper.SKIP:
            self.available_markets = available_markets 
        if disc_number is not APIHelper.SKIP:
            self.disc_number = disc_number 
        if duration_ms is not APIHelper.SKIP:
            self.duration_ms = duration_ms 
        if explicit is not APIHelper.SKIP:
            self.explicit = explicit 
        if external_urls is not APIHelper.SKIP:
            self.external_urls = external_urls 
        if href is not APIHelper.SKIP:
            self.href = href 
        if id is not APIHelper.SKIP:
            self.id = id 
        if is_playable is not APIHelper.SKIP:
            self.is_playable = is_playable 
        if linked_from is not APIHelper.SKIP:
            self.linked_from = linked_from 
        if restrictions is not APIHelper.SKIP:
            self.restrictions = restrictions 
        if name is not APIHelper.SKIP:
            self.name = name 
        if preview_url is not APIHelper.SKIP:
            self.preview_url = preview_url 
        if track_number is not APIHelper.SKIP:
            self.track_number = track_number 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if uri is not APIHelper.SKIP:
            self.uri = uri 
        if is_local is not APIHelper.SKIP:
            self.is_local = is_local 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        artists = None
        if dictionary.get('artists') is not None:
            artists = [SimplifiedArtistObject.from_dictionary(x) for x in dictionary.get('artists')]
        else:
            artists = APIHelper.SKIP
        available_markets = dictionary.get("available_markets") if dictionary.get("available_markets") else APIHelper.SKIP
        disc_number = dictionary.get("disc_number") if dictionary.get("disc_number") else APIHelper.SKIP
        duration_ms = dictionary.get("duration_ms") if dictionary.get("duration_ms") else APIHelper.SKIP
        explicit = dictionary.get("explicit") if "explicit" in dictionary.keys() else APIHelper.SKIP
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if 'external_urls' in dictionary.keys() else APIHelper.SKIP
        href = dictionary.get("href") if dictionary.get("href") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        is_playable = dictionary.get("is_playable") if "is_playable" in dictionary.keys() else APIHelper.SKIP
        linked_from = LinkedTrackObject.from_dictionary(dictionary.get('linked_from')) if 'linked_from' in dictionary.keys() else APIHelper.SKIP
        restrictions = TrackRestrictionObject.from_dictionary(dictionary.get('restrictions')) if 'restrictions' in dictionary.keys() else APIHelper.SKIP
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        preview_url = dictionary.get("preview_url") if "preview_url" in dictionary.keys() else APIHelper.SKIP
        track_number = dictionary.get("track_number") if dictionary.get("track_number") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        uri = dictionary.get("uri") if dictionary.get("uri") else APIHelper.SKIP
        is_local = dictionary.get("is_local") if "is_local" in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(artists,
                   available_markets,
                   disc_number,
                   duration_ms,
                   explicit,
                   external_urls,
                   href,
                   id,
                   is_playable,
                   linked_from,
                   restrictions,
                   name,
                   preview_url,
                   track_number,
                   mtype,
                   uri,
                   is_local)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'artists={(self.artists if hasattr(self, "artists") else None)!r}, '
                f'available_markets={(self.available_markets if hasattr(self, "available_markets") else None)!r}, '
                f'disc_number={(self.disc_number if hasattr(self, "disc_number") else None)!r}, '
                f'duration_ms={(self.duration_ms if hasattr(self, "duration_ms") else None)!r}, '
                f'explicit={(self.explicit if hasattr(self, "explicit") else None)!r}, '
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!r}, '
                f'href={(self.href if hasattr(self, "href") else None)!r}, '
                f'id={(self.id if hasattr(self, "id") else None)!r}, '
                f'is_playable={(self.is_playable if hasattr(self, "is_playable") else None)!r}, '
                f'linked_from={(self.linked_from if hasattr(self, "linked_from") else None)!r}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'preview_url={(self.preview_url if hasattr(self, "preview_url") else None)!r}, '
                f'track_number={(self.track_number if hasattr(self, "track_number") else None)!r}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!r}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!r}, '
                f'is_local={(self.is_local if hasattr(self, "is_local") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'artists={(self.artists if hasattr(self, "artists") else None)!s}, '
                f'available_markets={(self.available_markets if hasattr(self, "available_markets") else None)!s}, '
                f'disc_number={(self.disc_number if hasattr(self, "disc_number") else None)!s}, '
                f'duration_ms={(self.duration_ms if hasattr(self, "duration_ms") else None)!s}, '
                f'explicit={(self.explicit if hasattr(self, "explicit") else None)!s}, '
                f'external_urls={(self.external_urls if hasattr(self, "external_urls") else None)!s}, '
                f'href={(self.href if hasattr(self, "href") else None)!s}, '
                f'id={(self.id if hasattr(self, "id") else None)!s}, '
                f'is_playable={(self.is_playable if hasattr(self, "is_playable") else None)!s}, '
                f'linked_from={(self.linked_from if hasattr(self, "linked_from") else None)!s}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'preview_url={(self.preview_url if hasattr(self, "preview_url") else None)!s}, '
                f'track_number={(self.track_number if hasattr(self, "track_number") else None)!s}, '
                f'mtype={(self.mtype if hasattr(self, "mtype") else None)!s}, '
                f'uri={(self.uri if hasattr(self, "uri") else None)!s}, '
                f'is_local={(self.is_local if hasattr(self, "is_local") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.configurations.global_configuration import GlobalConfiguration
from apimatic_core.decorators.lazy_property import LazyProperty
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Configuration
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.base_controller import BaseController
from spotifywebapiwithfixesandimprovementsfromsonallux.configuration import Environment
from spotifywebapiwithfixesandimprovementsfromsonallux.http.auth.o_auth_2 import OAuth2
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.albums_controller\
    import AlbumsController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.artists_controller\
    import ArtistsController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.audiobooks_controller\
    import AudiobooksController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.categories_controller\
    import CategoriesController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.chapters_controller\
    import ChaptersController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.episodes_controller\
    import EpisodesController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.genres_controller\
    import GenresController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.markets_controller\
    import MarketsController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.player_controller\
    import PlayerController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.playlists_controller\
    import PlaylistsController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.search_controller\
    import SearchController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.shows_controller\
    import ShowsController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.tracks_controller\
    import TracksController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.users_controller\
    import UsersController
from spotifywebapiwithfixesandimprovementsfromsonallux.controllers.o_auth_authorization_controller\
    import OAuthAuthorizationController


class SpotifywebapiwithfixesandimprovementsfromsonalluxClient(object):
    @LazyProperty
    def albums(self):
        return AlbumsController(self.global_configuration)

    @LazyProperty
    def artists(self):
        return ArtistsController(self.global_configuration)

    @LazyProperty
    def audiobooks(self):
        return AudiobooksController(self.global_configuration)

    @LazyProperty
    def categories(self):
        return CategoriesController(self.global_configuration)

    @LazyProperty
    def chapters(self):
        return ChaptersController(self.global_configuration)

    @LazyProperty
    def episodes(self):
        return EpisodesController(self.global_configuration)

    @LazyProperty
    def genres(self):
        return GenresController(self.global_configuration)

    @LazyProperty
    def markets(self):
        return MarketsController(self.global_configuration)

    @LazyProperty
    def player(self):
        return PlayerController(self.global_configuration)

    @LazyProperty
    def playlists(self):
        return PlaylistsController(self.global_configuration)

    @LazyProperty
    def search(self):
        return SearchController(self.global_configuration)

    @LazyProperty
    def shows(self):
        return ShowsController(self.global_configuration)

    @LazyProperty
    def tracks(self):
        return TracksController(self.global_configuration)

    @LazyProperty
    def users(self):
        return UsersController(self.global_configuration)

    @LazyProperty
    def o_auth_authorization(self):
        return OAuthAuthorizationController(self.global_configuration)

    @property
    def oauth_2_0(self):
        return self.auth_managers['oauth_2_0']

    def __init__(self, http_client_instance=None,
                 override_http_client_configuration=False, http_call_back=None,
                 timeout=60, max_retries=0, backoff_factor=2,
                 retry_statuses=None, retry_methods=None,
                 environment=Environment.PRODUCTION, o_auth_client_id=None,
                 o_auth_client_secret=None, o_auth_redirect_uri=None,
                 o_auth_token=None, o_auth_scopes=None,
                 authorization_code_auth_credentials=None, config=None):
        self.config = config or Configuration(
            http_client_instance=http_client_instance,
            override_http_client_configuration=override_http_client_configuration,
            http_call_back=http_call_back, timeout=timeout,
            max_retries=max_retries, backoff_factor=backoff_factor,
            retry_statuses=retry_statuses, retry_methods=retry_methods,
            environment=environment, o_auth_client_id=o_auth_client_id,
            o_auth_client_secret=o_auth_client_secret,
            o_auth_redirect_uri=o_auth_redirect_uri, o_auth_token=o_auth_token,
            o_auth_scopes=o_auth_scopes,
            authorization_code_auth_credentials=authorization_code_auth_credentials)

        self.global_configuration = GlobalConfiguration(self.config)\
            .global_errors(BaseController.global_errors())\
            .base_uri_executor(self.config.get_base_uri)\
            .user_agent(BaseController.user_agent(), BaseController.user_agent_parameters())

        self.auth_managers = {key: None for key in ['oauth_2_0']}
        self.auth_managers['oauth_2_0'] = OAuth2(
            self.config.authorization_code_auth_credentials,
            self.global_configuration)
        self.global_configuration = self.global_configuration.auth_managers(self.auth_managers)




# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper


class SegmentObject(object):

    """Implementation of the 'SegmentObject' model.

    Attributes:
        start (float): The starting point (in seconds) of the segment.
        duration (float): The duration (in seconds) of the segment.
        confidence (float): The confidence, from 0.0 to 1.0, of the
            reliability of the segmentation. Segments of the song which are
            difficult to logically segment (e.g: noise) may correspond to low
            values in this field.
        loudness_start (float): The onset loudness of the segment in decibels
            (dB). Combined with `loudness_max` and `loudness_max_time`, these
            components can be used to describe the "attack" of the segment.
        loudness_max (float): The peak loudness of the segment in decibels
            (dB). Combined with `loudness_start` and `loudness_max_time`,
            these components can be used to describe the "attack" of the
            segment.
        loudness_max_time (float): The segment-relative offset of the segment
            peak loudness in seconds. Combined with `loudness_start` and
            `loudness_max`, these components can be used to desctibe the
            "attack" of the segment.
        loudness_end (float): The offset loudness of the segment in decibels
            (dB). This value should be equivalent to the loudness_start of the
            following segment.
        pitches (List[float]): Pitch content is given by a “chroma” vector,
            corresponding to the 12 pitch classes C, C#, D to B, with values
            ranging from 0 to 1 that describe the relative dominance of every
            pitch in the chromatic scale. For example a C Major chord would
            likely be represented by large values of C, E and G (i.e. classes
            0, 4, and 7).  Vectors are normalized to 1 by their strongest
            dimension, therefore noisy sounds are likely represented by values
            that are all close to 1, while pure tones are described by one
            value at 1 (the pitch) and others near 0. As can be seen below,
            the 12 vector indices are a combination of low-power spectrum
            values at their respective pitch frequencies. ![pitch
            vector](https://developer.spotify.com/assets/audio/Pitch_vector.png
            )
        timbre (List[float]): Timbre is the quality of a musical note or sound
            that distinguishes different types of musical instruments, or
            voices. It is a complex notion also referred to as sound color,
            texture, or tone quality, and is derived from the shape of a
            segment’s spectro-temporal surface, independently of pitch and
            loudness. The timbre feature is a vector that includes 12
            unbounded values roughly centered around 0. Those values are high
            level abstractions of the spectral surface, ordered by degree of
            importance.  For completeness however, the first dimension
            represents the average loudness of the segment; second emphasizes
            brightness; third is more closely correlated to the flatness of a
            sound; fourth to sounds with a stronger attack; etc. See an image
            below representing the 12 basis functions (i.e. template
            segments). ![timbre basis
            functions](https://developer.spotify.com/assets/audio/Timbre_basis_
            functions.png)  The actual timbre of the segment is best described
            as a linear combination of these 12 basis functions weighted by
            the coefficient values: timbre = c1 x b1 + c2 x b2 + ... + c12 x
            b12, where c1 to c12 represent the 12 coefficients and b1 to b12
            the 12 basis functions as displayed below. Timbre vectors are best
            used in comparison with each other.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "start": 'start',
        "duration": 'duration',
        "confidence": 'confidence',
        "loudness_start": 'loudness_start',
        "loudness_max": 'loudness_max',
        "loudness_max_time": 'loudness_max_time',
        "loudness_end": 'loudness_end',
        "pitches": 'pitches',
        "timbre": 'timbre'
    }

    _optionals = [
        'start',
        'duration',
        'confidence',
        'loudness_start',
        'loudness_max',
        'loudness_max_time',
        'loudness_end',
        'pitches',
        'timbre',
    ]

    def __init__(self,
                 start=APIHelper.SKIP,
                 duration=APIHelper.SKIP,
                 confidence=APIHelper.SKIP,
                 loudness_start=APIHelper.SKIP,
                 loudness_max=APIHelper.SKIP,
                 loudness_max_time=APIHelper.SKIP,
                 loudness_end=APIHelper.SKIP,
                 pitches=APIHelper.SKIP,
                 timbre=APIHelper.SKIP):
        """Constructor for the SegmentObject class"""

        # Initialize members of the class
        if start is not APIHelper.SKIP:
            self.start = start 
        if duration is not APIHelper.SKIP:
            self.duration = duration 
        if confidence is not APIHelper.SKIP:
            self.confidence = confidence 
        if loudness_start is not APIHelper.SKIP:
            self.loudness_start = loudness_start 
        if loudness_max is not APIHelper.SKIP:
            self.loudness_max = loudness_max 
        if loudness_max_time is not APIHelper.SKIP:
            self.loudness_max_time = loudness_max_time 
        if loudness_end is not APIHelper.SKIP:
            self.loudness_end = loudness_end 
        if pitches is not APIHelper.SKIP:
            self.pitches = pitches 
        if timbre is not APIHelper.SKIP:
            self.timbre = timbre 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        start = dictionary.get("start") if dictionary.get("start") else APIHelper.SKIP
        duration = dictionary.get("duration") if dictionary.get("duration") else APIHelper.SKIP
        confidence = dictionary.get("confidence") if dictionary.get("confidence") else APIHelper.SKIP
        loudness_start = dictionary.get("loudness_start") if dictionary.get("loudness_start") else APIHelper.SKIP
        loudness_max = dictionary.get("loudness_max") if dictionary.get("loudness_max") else APIHelper.SKIP
        loudness_max_time = dictionary.get("loudness_max_time") if dictionary.get("loudness_max_time") else APIHelper.SKIP
        loudness_end = dictionary.get("loudness_end") if dictionary.get("loudness_end") else APIHelper.SKIP
        pitches = dictionary.get("pitches") if dictionary.get("pitches") else APIHelper.SKIP
        timbre = dictionary.get("timbre") if dictionary.get("timbre") else APIHelper.SKIP
        # Return an object of this model
        return cls(start,
                   duration,
                   confidence,
                   loudness_start,
                   loudness_max,
                   loudness_max_time,
                   loudness_end,
                   pitches,
                   timbre)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'start={(self.start if hasattr(self, "start") else None)!r}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!r}, '
                f'confidence={(self.confidence if hasattr(self, "confidence") else None)!r}, '
                f'loudness_start={(self.loudness_start if hasattr(self, "loudness_start") else None)!r}, '
                f'loudness_max={(self.loudness_max if hasattr(self, "loudness_max") else None)!r}, '
                f'loudness_max_time={(self.loudness_max_time if hasattr(self, "loudness_max_time") else None)!r}, '
                f'loudness_end={(self.loudness_end if hasattr(self, "loudness_end") else None)!r}, '
                f'pitches={(self.pitches if hasattr(self, "pitches") else None)!r}, '
                f'timbre={(self.timbre if hasattr(self, "timbre") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'start={(self.start if hasattr(self, "start") else None)!s}, '
                f'duration={(self.duration if hasattr(self, "duration") else None)!s}, '
                f'confidence={(self.confidence if hasattr(self, "confidence") else None)!s}, '
                f'loudness_start={(self.loudness_start if hasattr(self, "loudness_start") else None)!s}, '
                f'loudness_max={(self.loudness_max if hasattr(self, "loudness_max") else None)!s}, '
                f'loudness_max_time={(self.loudness_max_time if hasattr(self, "loudness_max_time") else None)!s}, '
                f'loudness_end={(self.loudness_end if hasattr(self, "loudness_end") else None)!s}, '
                f'pitches={(self.pitches if hasattr(self, "pitches") else None)!s}, '
                f'timbre={(self.timbre if hasattr(self, "timbre") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_artist_object import PagingArtistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_playlist_object import PagingPlaylistObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_simplified_album_object import PagingSimplifiedAlbumObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_simplified_audiobook_object import PagingSimplifiedAudiobookObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_simplified_episode_object import PagingSimplifiedEpisodeObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_simplified_show_object import PagingSimplifiedShowObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.paging_track_object import PagingTrackObject


class SearchItems(object):

    """Implementation of the 'SearchItems' model.

    Attributes:
        tracks (PagingTrackObject): The model property of type
            PagingTrackObject.
        artists (PagingArtistObject): The model property of type
            PagingArtistObject.
        albums (PagingSimplifiedAlbumObject): The model property of type
            PagingSimplifiedAlbumObject.
        playlists (PagingPlaylistObject): The model property of type
            PagingPlaylistObject.
        shows (PagingSimplifiedShowObject): The model property of type
            PagingSimplifiedShowObject.
        episodes (PagingSimplifiedEpisodeObject): The model property of type
            PagingSimplifiedEpisodeObject.
        audiobooks (PagingSimplifiedAudiobookObject): The model property of
            type PagingSimplifiedAudiobookObject.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tracks": 'tracks',
        "artists": 'artists',
        "albums": 'albums',
        "playlists": 'playlists',
        "shows": 'shows',
        "episodes": 'episodes',
        "audiobooks": 'audiobooks'
    }

    _optionals = [
        'tracks',
        'artists',
        'albums',
        'playlists',
        'shows',
        'episodes',
        'audiobooks',
    ]

    def __init__(self,
                 tracks=APIHelper.SKIP,
                 artists=APIHelper.SKIP,
                 albums=APIHelper.SKIP,
                 playlists=APIHelper.SKIP,
                 shows=APIHelper.SKIP,
                 episodes=APIHelper.SKIP,
                 audiobooks=APIHelper.SKIP):
        """Constructor for the SearchItems class"""

        # Initialize members of the class
        if tracks is not APIHelper.SKIP:
            self.tracks = tracks 
        if artists is not APIHelper.SKIP:
            self.artists = artists 
        if albums is not APIHelper.SKIP:
            self.albums = albums 
        if playlists is not APIHelper.SKIP:
            self.playlists = playlists 
        if shows is not APIHelper.SKIP:
            self.shows = shows 
        if episodes is not APIHelper.SKIP:
            self.episodes = episodes 
        if audiobooks is not APIHelper.SKIP:
            self.audiobooks = audiobooks 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        tracks = PagingTrackObject.from_dictionary(dictionary.get('tracks')) if 'tracks' in dictionary.keys() else APIHelper.SKIP
        artists = PagingArtistObject.from_dictionary(dictionary.get('artists')) if 'artists' in dictionary.keys() else APIHelper.SKIP
        albums = PagingSimplifiedAlbumObject.from_dictionary(dictionary.get('albums')) if 'albums' in dictionary.keys() else APIHelper.SKIP
        playlists = PagingPlaylistObject.from_dictionary(dictionary.get('playlists')) if 'playlists' in dictionary.keys() else APIHelper.SKIP
        shows = PagingSimplifiedShowObject.from_dictionary(dictionary.get('shows')) if 'shows' in dictionary.keys() else APIHelper.SKIP
        episodes = PagingSimplifiedEpisodeObject.from_dictionary(dictionary.get('episodes')) if 'episodes' in dictionary.keys() else APIHelper.SKIP
        audiobooks = PagingSimplifiedAudiobookObject.from_dictionary(dictionary.get('audiobooks')) if 'audiobooks' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(tracks,
                   artists,
                   albums,
                   playlists,
                   shows,
                   episodes,
                   audiobooks)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'tracks={(self.tracks if hasattr(self, "tracks") else None)!r}, '
                f'artists={(self.artists if hasattr(self, "artists") else None)!r}, '
                f'albums={(self.albums if hasattr(self, "albums") else None)!r}, '
                f'playlists={(self.playlists if hasattr(self, "playlists") else None)!r}, '
                f'shows={(self.shows if hasattr(self, "shows") else None)!r}, '
                f'episodes={(self.episodes if hasattr(self, "episodes") else None)!r}, '
                f'audiobooks={(self.audiobooks if hasattr(self, "audiobooks") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'tracks={(self.tracks if hasattr(self, "tracks") else None)!s}, '
                f'artists={(self.artists if hasattr(self, "artists") else None)!s}, '
                f'albums={(self.albums if hasattr(self, "albums") else None)!s}, '
                f'playlists={(self.playlists if hasattr(self, "playlists") else None)!s}, '
                f'shows={(self.shows if hasattr(self, "shows") else None)!s}, '
                f'episodes={(self.episodes if hasattr(self, "episodes") else None)!s}, '
                f'audiobooks={(self.audiobooks if hasattr(self, "audiobooks") else None)!s})')



# -*- coding: utf-8 -*-

"""
spotifywebapiwithfixesandimprovementsfromsonallux

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from spotifywebapiwithfixesandimprovementsfromsonallux.api_helper import APIHelper
from spotifywebapiwithfixesandimprovementsfromsonallux.models.chapter_restriction_object import ChapterRestrictionObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.external_url_object import ExternalUrlObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.image_object import ImageObject
from spotifywebapiwithfixesandimprovementsfromsonallux.models.resume_point_object import ResumePointObject


class ChapterBase(object):

    """Implementation of the 'ChapterBase' model.

    Attributes:
        audio_preview_url (str): A URL to a 30 second preview (MP3 format) of
            the chapter. `null` if not available.
        available_markets (List[str]): A list of the countries in which the
            chapter can be played, identified by their [ISO 3166-1
            alpha-2](http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2) code.
        chapter_number (int): The number of the chapter
        description (str): A description of the chapter. HTML tags are
            stripped away from this field, use `html_description` field in
            case HTML tags are needed.
        html_description (str): A description of the chapter. This field may
            contain HTML tags.
        duration_ms (int): The chapter length in milliseconds.
        explicit (bool): Whether or not the chapter has explicit content (true
            = yes it does; false = no it does not OR unknown).
        external_urls (ExternalUrlObject): External URLs for this chapter.
        href (str): A link to the Web API endpoint providing full details of
            the chapter.
        id (str): The [Spotify
            ID](/documentation/web-api/concepts/spotify-uris-ids) for the
            chapter.
        images (List[ImageObject]): The cover art for the chapter in various
            sizes, widest first.
        is_playable (bool): True if the chapter is playable in the given
            market. Otherwise false.
        languages (List[str]): A list of the languages used in the chapter,
            identified by their [ISO
            639-1](https://en.wikipedia.org/wiki/ISO_639) code.
        name (str): The name of the chapter.
        release_date (str): The date the chapter was first released, for
            example `"1981-12-15"`. Depending on the precision, it might be
            shown as `"1981"` or `"1981-12"`.
        release_date_precision (ReleaseDatePrecisionEnum): The precision with
            which `release_date` value is known.
        resume_point (ResumePointObject): The user's most recent position in
            the chapter. Set if the supplied access token is a user token and
            has the scope 'user-read-playback-position'.
        mtype (str): The object type.
        uri (str): The [Spotify
            URI](/documentation/web-api/concepts/spotify-uris-ids) for the
            chapter.
        restrictions (ChapterRestrictionObject): Included in the response when
            a content restriction is applied.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "audio_preview_url": 'audio_preview_url',
        "chapter_number": 'chapter_number',
        "description": 'description',
        "html_description": 'html_description',
        "duration_ms": 'duration_ms',
        "explicit": 'explicit',
        "external_urls": 'external_urls',
        "href": 'href',
        "id": 'id',
        "images": 'images',
        "is_playable": 'is_playable',
        "languages": 'languages',
        "name": 'name',
        "release_date": 'release_date',
        "release_date_precision": 'release_date_precision',
        "mtype": 'type',
        "uri": 'uri',
        "available_markets": 'available_markets',
        "resume_point": 'resume_point',
        "restrictions": 'restrictions'
    }

    _optionals = [
        'available_markets',
        'resume_point',
        'restrictions',
    ]

    _nullables = [
        'audio_preview_url',
    ]

    def __init__(self,
                 audio_preview_url=None,
                 chapter_number=None,
                 description=None,
                 html_description=None,
                 duration_ms=None,
                 explicit=None,
                 external_urls=None,
                 href=None,
                 id=None,
                 images=None,
                 is_playable=None,
                 languages=None,
                 name=None,
                 release_date=None,
                 release_date_precision=None,
                 uri=None,
                 available_markets=APIHelper.SKIP,
                 resume_point=APIHelper.SKIP,
                 restrictions=APIHelper.SKIP):
        """Constructor for the ChapterBase class"""

        # Initialize members of the class
        self.audio_preview_url = audio_preview_url 
        if available_markets is not APIHelper.SKIP:
            self.available_markets = available_markets 
        self.chapter_number = chapter_number 
        self.description = description 
        self.html_description = html_description 
        self.duration_ms = duration_ms 
        self.explicit = explicit 
        self.external_urls = external_urls 
        self.href = href 
        self.id = id 
        self.images = images 
        self.is_playable = is_playable 
        self.languages = languages 
        self.name = name 
        self.release_date = release_date 
        self.release_date_precision = release_date_precision 
        if resume_point is not APIHelper.SKIP:
            self.resume_point = resume_point 
        self.mtype = 'episode' 
        self.uri = uri 
        if restrictions is not APIHelper.SKIP:
            self.restrictions = restrictions 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        audio_preview_url = dictionary.get("audio_preview_url") if dictionary.get("audio_preview_url") else None
        chapter_number = dictionary.get("chapter_number") if dictionary.get("chapter_number") else None
        description = dictionary.get("description") if dictionary.get("description") else None
        html_description = dictionary.get("html_description") if dictionary.get("html_description") else None
        duration_ms = dictionary.get("duration_ms") if dictionary.get("duration_ms") else None
        explicit = dictionary.get("explicit") if "explicit" in dictionary.keys() else None
        external_urls = ExternalUrlObject.from_dictionary(dictionary.get('external_urls')) if dictionary.get('external_urls') else None
        href = dictionary.get("href") if dictionary.get("href") else None
        id = dictionary.get("id") if dictionary.get("id") else None
        images = None
        if dictionary.get('images') is not None:
            images = [ImageObject.from_dictionary(x) for x in dictionary.get('images')]
        is_playable = dictionary.get("is_playable") if "is_playable" in dictionary.keys() else None
        languages = dictionary.get("languages") if dictionary.get("languages") else None
        name = dictionary.get("name") if dictionary.get("name") else None
        release_date = dictionary.get("release_date") if dictionary.get("release_date") else None
        release_date_precision = dictionary.get("release_date_precision") if dictionary.get("release_date_precision") else None
        uri = dictionary.get("uri") if dictionary.get("uri") else None
        available_markets = dictionary.get("available_markets") if dictionary.get("available_markets") else APIHelper.SKIP
        resume_point = ResumePointObject.from_dictionary(dictionary.get('resume_point')) if 'resume_point' in dictionary.keys() else APIHelper.SKIP
        restrictions = ChapterRestrictionObject.from_dictionary(dictionary.get('restrictions')) if 'restrictions' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(audio_preview_url,
                   chapter_number,
                   description,
                   html_description,
                   duration_ms,
                   explicit,
                   external_urls,
                   href,
                   id,
                   images,
                   is_playable,
                   languages,
                   name,
                   release_date,
                   release_date_precision,
                   uri,
                   available_markets,
                   resume_point,
                   restrictions)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'audio_preview_url={self.audio_preview_url!r}, '
                f'available_markets={(self.available_markets if hasattr(self, "available_markets") else None)!r}, '
                f'chapter_number={self.chapter_number!r}, '
                f'description={self.description!r}, '
                f'html_description={self.html_description!r}, '
                f'duration_ms={self.duration_ms!r}, '
                f'explicit={self.explicit!r}, '
                f'external_urls={self.external_urls!r}, '
                f'href={self.href!r}, '
                f'id={self.id!r}, '
                f'images={self.images!r}, '
                f'is_playable={self.is_playable!r}, '
                f'languages={self.languages!r}, '
                f'name={self.name!r}, '
                f'release_date={self.release_date!r}, '
                f'release_date_precision={self.release_date_precision!r}, '
                f'resume_point={(self.resume_point if hasattr(self, "resume_point") else None)!r}, '
                f'mtype={self.mtype!r}, '
                f'uri={self.uri!r}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'audio_preview_url={self.audio_preview_url!s}, '
                f'available_markets={(self.available_markets if hasattr(self, "available_markets") else None)!s}, '
                f'chapter_number={self.chapter_number!s}, '
                f'description={self.description!s}, '
                f'html_description={self.html_description!s}, '
                f'duration_ms={self.duration_ms!s}, '
                f'explicit={self.explicit!s}, '
                f'external_urls={self.external_urls!s}, '
                f'href={self.href!s}, '
                f'id={self.id!s}, '
                f'images={self.images!s}, '
                f'is_playable={self.is_playable!s}, '
                f'languages={self.languages!s}, '
                f'name={self.name!s}, '
                f'release_date={self.release_date!s}, '
                f'release_date_precision={self.release_date_precision!s}, '
                f'resume_point={(self.resume_point if hasattr(self, "resume_point") else None)!s}, '
                f'mtype={self.mtype!s}, '
                f'uri={self.uri!s}, '
                f'restrictions={(self.restrictions if hasattr(self, "restrictions") else None)!s})')
